"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var ramda_1 = __importDefault(require("ramda"));
var es2015_1 = __importDefault(require("estel-estree-builder/generated/es2015"));
var result_1 = __importDefault(require("folktale/result"));
var path_1 = __importDefault(require("path"));
var tcomb_1 = require("./tcomb");
var utils_1 = require("./utils");
var acorn_1 = require("./acorn");
var node_ops_1 = require("./node-ops");
var js_gen_1 = require("./js-gen");
var tree_builders_1 = require("./tree-builders");
exports.generateImports = function (componentName) {
    var namedImports = {
        react: ['Component'],
    };
    var defaultImports = {
        'prop-types': 'PropTypes',
        react: 'React',
    };
    defaultImports["./" + componentName + ".scss"] = 'styles';
    return tree_builders_1.buildImportStmts([namedImports], [defaultImports]);
};
var generateProps = function (className, attributeNodes) {
    var objProps = attributeNodes.map(function (a) {
        return es2015_1.default.property(es2015_1.default.identifier(a.name.name), tree_builders_1.dot(tree_builders_1.dot(es2015_1.default.identifier('PropTypes'), 'object'), 'isRequired'), 'init');
    });
    return tree_builders_1.assign(tree_builders_1.dot(es2015_1.default.identifier(className), 'propTypes'), es2015_1.default.objectExpression(objProps));
};
var generateRender = function (className, attributeNodes) {
    var propVars = attributeNodes.map(function (a) {
        var propName = es2015_1.default.identifier(ramda_1.default.path(['name', 'name'], a));
        return es2015_1.default.property(propName, propName, "init", false, true);
    });
    var objPattern = es2015_1.default.objectPattern(propVars);
    var thisDotProps = tree_builders_1.dot(es2015_1.default.thisExpression(), 'props');
    return tree_builders_1.fnArgs([], es2015_1.default.variableDeclaration([es2015_1.default.variableDeclarator(objPattern, thisDotProps)], 'const'), es2015_1.default.returnStatement(es2015_1.default.identifier("(\n      <div>\n        Here is a '" + className + "'\n      </div>\n    )")));
};
exports.generateComponents = function (jsxNode) {
    var className = ramda_1.default.path(['name', 'name'], jsxNode);
    if (!utils_1.validComponentName(className)) {
        return result_1.default.Error("The element name '" + className + "' is not valid for a component");
    }
    var classDecl = tree_builders_1.buildClass(className, 'Component', {
        render: generateRender(className, jsxNode.attributes),
    });
    var defProps = generateProps(className, jsxNode.attributes);
    var defExport = tree_builders_1.raw("export default " + className);
    return result_1.default.Ok([classDecl, defProps, defExport]);
};
exports.importNewComponent = function (nodes, compName, sourcePath) {
    var newImportPath = path_1.default.join(sourcePath, 'components', compName, compName);
    var newImportStmt = "import " + compName + " from '" + newImportPath + "'";
    var importNodes = nodes.filter(ramda_1.default.propEq("type", "ImportDeclaration"));
    var lastImport = importNodes.slice(-1)[0];
    if (lastImport !== undefined) {
        var alreadyImported = ramda_1.default.find(function (n) {
            return ramda_1.default.path(['source', 'value'], n) === newImportPath;
        }, importNodes);
        if (alreadyImported) {
            return [];
        }
        else {
            return [{ lineNumber: lastImport.loc.end.line + 1, content: newImportStmt }];
        }
    }
    else {
        return [{ lineNumber: 0, content: newImportStmt }];
    }
};
exports.generateComponent = function (inputText, point, sourcePath) {
    var tree = acorn_1.parse(inputText);
    node_ops_1.printNode(tree);
    var jsxBlock = node_ops_1.searchByLocationAndType(tree, point, 'JSXElement');
    if (!jsxBlock) {
        throw new Error("Could not locate a JSX snippet near point " + point);
    }
    var jsxOpening = jsxBlock.openingElement;
    var componentName = jsxOpening.name.name;
    var componentDefinitionCode = exports.generateComponents(jsxOpening);
    return componentDefinitionCode.map(function (c) {
        var changesToCaller = exports.importNewComponent(tree.body, componentName, sourcePath);
        var importStmts = exports.generateImports(componentName);
        var ast = importStmts.concat(c);
        return tcomb_1.ComponentGeneration({
            ast: ast,
            changesToCaller: changesToCaller,
            componentName: componentName,
            content: js_gen_1.genJsList(ast),
        });
    });
};
//# sourceMappingURL=react-content.js.map